# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Thesis ID provider."""

from __future__ import absolute_import, print_function

from nr_common.providers import NRIdProvider


class NRThesesIdProvider(NRIdProvider):
    """Thesss identifier provider."""

    pid_type = 'nrthe'
    """Type of persistent identifier."""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CIS UCT Prague.
#
# CIS theses repository is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Flask extension for CIS theses repository."""

from __future__ import absolute_import, print_function

import logging

from nr_theses.api import ThesisAPI
from . import config

log = logging.getLogger('nr-theses')


class NRTheses(object):
    """CIS theses repository extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)
        app.extensions['nr-theses'] = ThesisAPI(app)

    def init_config(self, app):
        """Initialize configuration.

        Override configuration variables with the values in this package.
        """
        app.config.setdefault('RECORDS_DRAFT_ENDPOINTS', {}).update(config.RECORDS_DRAFT_ENDPOINTS)
        app.config.setdefault('RECORDS_REST_FACETS', {}).update(config.RECORDS_REST_FACETS)

        app.config.setdefault('RECORDS_REST_SORT_OPTIONS', {}).update(
            config.RECORDS_REST_SORT_OPTIONS)

        app.config.setdefault('RECORDS_REST_DEFAULT_SORT', {}).update(
            config.RECORDS_REST_DEFAULT_SORT)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CERN.
#
# My site is free software; you can redistribute it and/or modify it under
# the terms of the MIT License; see LICENSE file for more details.

"""Schemas for marshmallow."""

from __future__ import absolute_import, print_function

from .json import ThesisMetadataSchemaV2

__all__ = ('ThesisMetadataSchemaV2',)



import uuid

from invenio_db import db
from invenio_indexer.api import RecordIndexer
from invenio_pidstore.errors import PIDDoesNotExistError
from invenio_pidstore.models import PIDStatus
from invenio_pidstore.models import PersistentIdentifier
from invenio_records import Record
from invenio_search import RecordsSearch
from sqlalchemy.orm.exc import NoResultFound

from nr_theses.record import PublishedThesisRecord


class ThesisSearch(RecordsSearch):
    pass


class ThesisAPI:

    def __init__(self, app):
        """
        API initialization.

        :param app: invenio application
        """
        self.app = app
        self.indexer = RecordIndexer()

    @staticmethod
    def create_draft_record(record: dict, pid_type=None, pid_value=None):
        if not pid_type:
            pid_type = "dnr"
        if not pid_value:
            pid_value = record["control_number"]
        id_ = uuid.uuid4()
        pid = PersistentIdentifier.create(
            pid_type,
            pid_value,
            pid_provider=None,
            object_type="rec",
            object_uuid=id_,
            status=PIDStatus.REGISTERED,
        )
        db_record = PublishedThesisRecord.create(record, id_=id_)
        return db_record

    @staticmethod
    def delete_draft_record(record: Record):
        record.delete()

    @staticmethod
    def get_record_by_id(pid_type, pid_value):
        try:
            existing_pid = PersistentIdentifier.get(pid_type, pid_value)
            try:
                existing_record = PublishedThesisRecord.get_record(id_=existing_pid.object_uuid)
            except NoResultFound:
                # check it if has not been deleted and salvage if so
                existing_record = PublishedThesisRecord.get_record(id_=existing_pid.object_uuid,
                                                                   with_deleted=True)
                existing_record = existing_record.revert(-1)
        except PIDDoesNotExistError:
            return
        except NoResultFound:  # pragma: no cover
            return
        return existing_record



import os

from flask import url_for
from invenio_records.api import Record
from oarepo_communities.converters import CommunityPIDValue
from oarepo_communities.proxies import current_oarepo_communities
from oarepo_communities.record import CommunityRecordMixin
from oarepo_fsm.mixins import FSMMixin
from oarepo_records_draft.record import InvalidRecordAllowedMixin, DraftRecordMixin
from oarepo_references.mixins import ReferenceEnabledRecordMixin
from oarepo_validate import SchemaKeepingRecordMixin, MarshmallowValidatedRecordMixin

from .constants import THESES_ALLOWED_SCHEMAS, THESES_PREFERRED_SCHEMA
from .marshmallow import ThesisMetadataSchemaV2

published_index_name = 'nr_theses-nr-theses-v1.0.0'
draft_index_name = 'draft-nr_theses-nr-theses-v1.0.0'
all_index_name = 'nr-all'

prefixed_published_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX',
                                               '') + published_index_name
prefixed_draft_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX', '') + draft_index_name
prefixed_all_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX', '') + all_index_name


class ThesisBaseRecord(SchemaKeepingRecordMixin,
                       MarshmallowValidatedRecordMixin,
                       ReferenceEnabledRecordMixin,
                       CommunityRecordMixin,
                       Record,
                       ):
    ALLOWED_SCHEMAS = THESES_ALLOWED_SCHEMAS
    PREFERRED_SCHEMA = THESES_PREFERRED_SCHEMA
    MARSHMALLOW_SCHEMA = ThesisMetadataSchemaV2


class PublishedThesisRecord(InvalidRecordAllowedMixin, ThesisBaseRecord):
    index_name = published_index_name

    @property
    def canonical_url(self):
        return url_for('invenio_records_rest.theses_item',
                       pid_value=CommunityPIDValue(
                           self['control_number'],
                           current_oarepo_communities.get_primary_community_field(self)),
                       _external=True)


class DraftThesisRecord(DraftRecordMixin, ThesisBaseRecord):
    index_name = draft_index_name

    @property
    def canonical_url(self):
        return url_for('invenio_records_rest.draft-theses_item',
                       pid_value=CommunityPIDValue(
                           self['control_number'],
                           current_oarepo_communities.get_primary_community_field(self)),
                       _external=True)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CIS UCT Prague.
#
# CIS theses repository is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Default configuration."""

from __future__ import absolute_import, print_function

from invenio_records_rest.utils import allow_all, deny_all
from oarepo_communities.links import community_record_links_factory
from oarepo_records_draft import DRAFT_IMPORTANT_FILTERS
from oarepo_records_draft.rest import DRAFT_IMPORTANT_FACETS

from nr_generic.config import FACETS, CURATOR_FACETS, CURATOR_FILTERS, FILTERS
from nr_theses.record import draft_index_name
from oarepo_multilingual import language_aware_text_term_facet, language_aware_text_terms_filter
from oarepo_ui.facets import translate_facets, term_facet
from oarepo_ui.filters import boolean_filter

from nr_theses.search import ThesisRecordsSearch

_ = lambda x: x

RECORDS_DRAFT_ENDPOINTS = {
    'theses': {
        'draft': 'draft-theses',
        'pid_type': 'nrthe',
        'pid_minter': 'nr_theses',
        'pid_fetcher': 'nr_theses',
        'default_endpoint_prefix': True,
        'max_result_window': 500000,
        'record_class': 'nr_theses.record:PublishedThesisRecord',
        'list_route': '/<community_id>/theses/',
        'item_route': f'/<commpid(nrthe,model="theses",record_class="nr_theses.record:PublishedThesisRecord"):pid_value>',

        'publish_permission_factory_imp': 'nr_common.permissions.publish_draft_object_permission_impl',
        'unpublish_permission_factory_imp': 'nr_common.permissions.unpublish_draft_object_permission_impl',
        'edit_permission_factory_imp': 'nr_common.permissions.update_object_permission_impl',
        'list_permission_factory_imp': allow_all,
        'read_permission_factory_imp': allow_all,
        'create_permission_factory_imp': deny_all,
        'update_permission_factory_imp': deny_all,
        'delete_permission_factory_imp': deny_all,
        'default_media_type': 'application/json',
        'links_factory_imp': community_record_links_factory,
        'search_class': ThesisRecordsSearch,
        # 'indexer_class': CommitingRecordIndexer,
        'files': dict(
            # Who can upload attachments to a draft dataset record
            put_file_factory=deny_all,
            # Who can download attachments from a draft dataset record
            get_file_factory=allow_all,
            # Who can delete attachments from a draft dataset record
            delete_file_factory=deny_all
        )

    },
    'draft-theses': {
        'pid_type': 'dnrthe',
        'record_class': 'nr_theses.record:DraftThesisRecord',
        'list_route': '/<community_id>/theses/draft/',
        'item_route': f'/<commpid(nrthe,model="theses/draft",record_class="nr_theses.record:DraftThesisRecord"):pid_value>',
        'search_index': draft_index_name,
        'links_factory_imp': community_record_links_factory,
        'search_class': ThesisRecordsSearch,
'search_serializers': {
            'application/json': 'oarepo_validate:json_search',
        },
        'record_serializers': {
            'application/json': 'oarepo_validate:json_response',
        },

        'create_permission_factory_imp': 'nr_common.permissions.create_draft_object_permission_impl',
        'update_permission_factory_imp': 'nr_common.permissions.update_draft_object_permission_impl',
        'read_permission_factory_imp': 'nr_common.permissions.read_draft_object_permission_impl',
        'delete_permission_factory_imp': 'nr_common.permissions.delete_draft_object_permission_impl',
        'list_permission_factory_imp': 'nr_common.permissions.list_draft_object_permission_impl',
        'record_loaders': {
            'application/json': 'oarepo_validate.json_files_loader',
            'application/json-patch+json': 'oarepo_validate.json_loader'
        },
        'files': dict(
            put_file_factory='nr_common.permissions.put_draft_file_permission_impl',
            get_file_factory='nr_common.permissions.get_draft_file_permission_impl',
            delete_file_factory='nr_common.permissions.delete_draft_file_permission_impl'
        )
    }
}

THESES_FILTERS = {
    _('defended'): boolean_filter('defended'),
    _('studyField'): language_aware_text_terms_filter('studyField.title', suffix=".keyword"),
    _('degreeGrantor'): language_aware_text_terms_filter('degreeGrantor.title', suffix=".keyword"),
    #     _('person'): terms_filter('person.keyword'),
    #     _('accessRights'): group_by_terms_filter('accessRights.title.en.raw', {
    #         "true": "open access",
    #         1: "open access",
    #         True: "open access",
    #         "1": "open access",
    #         False: ["embargoed access", "restricted access", "metadata only access"],
    #         0: ["embargoed access", "restricted access", "metadata only access"],
    #         "false": ["embargoed access", "restricted access", "metadata only access"],
    #         "0": ["embargoed access", "restricted access", "metadata only access"],
    #     }),
    #     _('resourceType'): language_aware_text_terms_filter('resourceType.title'),
    #     _('keywords'): language_aware_text_terms_filter('keywords'),
    #     _('subject'): language_aware_text_terms_filter('subjectAll'),
    #     _('language'): language_aware_text_terms_filter('language.title'),
    #     _('date'): range_filter('dateAll.date'),
    #     _('dateIssued'): range_filter('dateIssued.date'),
    #     _('dateDefended'): range_filter('dateDefended.date'),
    #     _('dateModified'): range_filter('dateModified.date'),
    #
}
#
# CURATOR_FILTERS = {
#     _('rights'): language_aware_text_terms_filter('rights.title'),
#     _('provider'): language_aware_text_terms_filter('provider.title'),
#     _('isGL'): boolean_filter('isGL')
# }
#
THESES_FACETS = {
    'defended': term_facet('defended'),
    'studyField': language_aware_text_term_facet('studyField.title', suffix=".keyword"),
    'degreeGrantor': language_aware_text_term_facet('degreeGrantor.title', suffix=".keyword"),
    # 'person': term_facet('person.keyword'),
    # 'accessRights': term_facet('accessRights.title.en.raw'),
    # 'resourceType': language_aware_text_term_facet('resourceType.title'),
    # 'keywords': language_aware_text_term_facet('keywords'),
    # 'subject': language_aware_text_term_facet('subjectAll'),
    # 'language': language_aware_text_term_facet('language.title'),
    # 'date': date_histogram_facet('dateAll.date'),
}
#
# CURATOR_FACETS = {
#     'rights': language_aware_text_term_facet('rights.title'),
#     'provider': language_aware_text_term_facet('provider.title'),
#     'dateIssued': date_histogram_facet('dateIssued.date'),
#     'dateDefended': date_histogram_facet('dateDefended.date'),
#     'dateModified': date_histogram_facet('dateModified.date'),
#     'isGL': term_facet('isGL'),
# }

RECORDS_REST_FACETS = {
    draft_index_name: {
        "aggs": translate_facets({**THESES_FACETS, **FACETS, **CURATOR_FACETS, **DRAFT_IMPORTANT_FACETS},
                                 label='{facet_key}',
                                 value='{value_key}'),
        "filters": {**THESES_FILTERS, **FILTERS, **CURATOR_FILTERS, **DRAFT_IMPORTANT_FILTERS}
    },
}

RECORDS_REST_SORT_OPTIONS = {
    draft_index_name: {
        'alphabetical': {
            'title': 'alphabetical',
            'fields': [
                'title.cs.raw'
            ],
            'default_order': 'asc',
            'order': 1
        },
        'best_match': {
            'title': 'Best match',
            'fields': ['_score'],
            'default_order': 'desc',
            'order': 1,
        }
    }
}

RECORDS_REST_DEFAULT_SORT = {
    draft_index_name: {
        'query': 'best_match',
        'noquery': 'best_match'
    }
}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nr_theses',
 'nr_theses.jsonschemas',
 'nr_theses.jsonschemas.nr_theses',
 'nr_theses.mapping_includes',
 'nr_theses.mapping_includes.v7',
 'nr_theses.mappings',
 'nr_theses.mappings.v7',
 'nr_theses.marshmallow']

package_data = \
{'': ['*'], 'nr_theses.mappings.v7': ['nr_theses/*']}

install_requires = \
['oarepo>=3.3.59,<4.0.0', 'techlib-nr-common>=3.0.0a44,<4.0.0']

entry_points = \
{'invenio_base.api_apps': ['theses = nr_theses:NRTheses'],
 'invenio_base.apps': ['theses = nr_theses:NRTheses'],
 'invenio_jsonschemas.schemas': ['nr_theses = nr_theses.jsonschemas'],
 'invenio_pidstore.fetchers': ['nr_theses = '
                               'nr_theses.fetchers:nr_theses_id_fetcher'],
 'invenio_pidstore.minters': ['nr_theses = '
                              'nr_theses.minters:nr_theses_id_minter'],
 'invenio_search.mappings': ['nr_theses = nr_theses.mappings'],
 'oarepo_mapping_includes': ['nr_theses = nr_theses.mapping_includes']}

setup_kwargs = {
    'name': 'techlib-nr-theses',
    'version': '1.0.0a3',
    'description': 'Czech National Repository theses data model.',
    'long_description': '# NR-Theses\n\n[![Build Status](https://travis-ci.org/Narodni-repozitar/nr-theses.svg?branch=master)](https://travis-ci.org/Narodni-repozitar/nr-theses)\n[![Coverage Status](https://coveralls.io/repos/github/Narodni-repozitar/nr-theses/badge.svg?branch=master)](https://coveralls.io/github/Narodni-repozitar/nr-theses?branch=master)\n\n\nDisclaimer: The library is part of the Czech National Repository, and therefore the README is written in Czech.\nGeneral libraries extending [Invenio](https://github.com/inveniosoftware) are concentrated under the [Oarepo\n namespace](https://github.com/oarepo).\n\n  ## Instalace\n\n Nejedná se o samostatně funkční knihovnu, proto potřebuje běžící Invenio a závislosti Oarepo.\n Knihovna se instaluje ze zdroje.\n\n ```bash\ngit clone git@github.com:Narodni-repozitar/nr-theses.git\ncd nr-common\npip install poetry\npoetry install\n```\n\nPro testování a/nebo samostané fungování knihovny je nutné instalovat tests z extras.\n\n```bash\npoetry install --extras tests\n```\n\n:warning: Pro instalaci se používá Manažer závilostí **Poetry** více infromací lze naleznout v\n[dokumentaci](https://python-poetry.org/docs/)\n\n## Účel\n\nKnihovna rozšiřuje [obecný metadatový model](https://github.com/Narodni-repozitar/nr-common)\no pole pro vysokoškolské závěrečné práce. Vysokoškolským pracím je přiřazen endpoint **/api/theses**. Knihovna\nposkytuje API pro CRUD operace pod proxy **nr_these**.\n\n## Použití\n\nBude dopsáno.\n',
    'author': 'Daniel Kopecký',
    'author_email': 'Daniel.Kopecky@techlib.cz',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from elasticsearch_dsl import Q
from elasticsearch_dsl.query import Bool
from oarepo_communities.constants import STATE_PUBLISHED, STATE_APPROVED
from oarepo_communities.search import CommunitySearch

from nr_common.permissions import (
    AUTHENTICATED_PERMISSION,
    COMMUNITY_MEMBER_PERMISSION,
    COMMUNITY_CURATOR_PERMISSION
)


class ThesisRecordsSearch(CommunitySearch):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._source = self._source = [
            'control_number', 'oarepo:validity.valid', 'oarepo:draft', 'title', 'dateIssued',
            'creator', 'resourceType', 'contributor', 'keywords', 'subject', 'abstract', 'state',
            '_administration.primaryCommunity',
            '_administration.communities',
        ]
        self._highlight['title.cs'] = {}
        self._highlight['title._'] = {}
        self._highlight['title.en'] = {}

    class Meta:
        doc_types = ['_doc']
        default_anonymous_filter = Q('term', _administration__state=STATE_PUBLISHED)
        default_authenticated_filter = Q('terms', state=[STATE_APPROVED, STATE_PUBLISHED])

        @staticmethod
        def default_filter_factory(search=None, **kwargs):
            if not AUTHENTICATED_PERMISSION.can() or not (COMMUNITY_MEMBER_PERMISSION(None).can() or COMMUNITY_CURATOR_PERMISSION(None).can()):
                # Anonymous or non-community members sees published community records only
                return Bool(must=[
                    ThesisRecordsSearch.Meta.default_anonymous_filter,
                    CommunitySearch.community_filter()])
            else:
                if COMMUNITY_CURATOR_PERMISSION(None).can():
                    # Curators can see all community records
                    return CommunitySearch.community_filter()

                # Community member sees both APPROVED and PUBLISHED community records only
                q = Bool(must=[
                    CommunitySearch.community_filter(),
                    ThesisRecordsSearch.Meta.default_authenticated_filter])
            return q



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CERN.
#
# My site is free software; you can redistribute it and/or modify it under
# the terms of the MIT License; see LICENSE file for more details.

"""JSON Schemas."""
import arrow
from invenio_records_rest.schemas.fields.datetime import DateString
from marshmallow import fields, validates, ValidationError
from nr_common.marshmallow import CommonMetadataSchemaV2
from nr_common.marshmallow.subschemas import TitledMixin, InstitutionsMixin
from oarepo_taxonomies.marshmallow import TaxonomyField

from nr_theses.marshmallow.subschemas import StudyFieldMixin


class ThesisMetadataSchemaV2(CommonMetadataSchemaV2):
    dateDefended = DateString(required=True)
    defended = fields.Boolean(required=True)
    degreeGrantor = TaxonomyField(mixins=[TitledMixin, InstitutionsMixin], required=True)
    studyField = TaxonomyField(name="studyField", mixins=[TitledMixin, StudyFieldMixin])

    @validates("dateDefended")
    def validate_date_range(self, value):
        date = arrow.get(value)
        min_ = arrow.get("1700-01-01")
        current_date = arrow.get()
        if date > current_date:
            raise ValidationError("Date cannot be in the future")
        if date < min_:
            raise ValidationError("Records older than from 1700 is not supported")
